#!/usr/bin/env python3
#-*- coding:utf-8 -*-

# Gery Casiez / Axel Antoine
# 2020

import json
from os import listdir
from os.path import join
import csv
import re
import time
import random
import argparse
from tools import MovingAverage
from tools.common import *
from tools.extra import *

pred = {}
predTechs = ["linear"]

delays = [0,1,2,3,4,5,6,7,8,9,10]
frequencies = [30, 40, 50, 60, 70, 80, 90, 100, 110, 120]

def computeOutputViewPortPos(s: [], outFreq: int) -> []:
    out = []

    # Compute output-ts and output-pos
    # For the first frametime, we randomly add a number of ms between 0 and 1/f
    r = random.randint(0, int(1/outFreq*1E3)) # in ms
    frametime = s[0]["timestamp"] + r * 1E3 # in micros

    MA_size = 2
    MA = MovingAverage.MovingAverage(MA_size)

    last_event = None

    i = 1
    while i < len(s):
        # Get the index of the last event before frametime
        while (i < len(s)) and (s[i]["timestamp"] < frametime) : # <= ?
            i += 1

        # make sure we handle event only once 
        if last_event != i:
            d = {}
            # frametime and position of the viewport at frametime
            d["output-ts"] = frametime
            d["output-pos"] = s[i-1]['y']

            # Interpolation of the viewport at frametime
            d["input-pos-interpol"] = interpolate_at_timestamp(s, frametime, d)
            d["timeToFrametime"] = (frametime - s[i-1]["timestamp"]) * 1E-3
            d["distToFrametime"] = abs(d["input-pos-interpol"] - d["output-pos"])

            # Moving average version Axel
            d["output-posMA"] = MA(d["output-pos"])
            introduced_latency_MA = 0.5 * (MA_size - 1) / outFreq * 1E3
            interpolation_timestamp_MA = s[i-1]["timestamp"] - introduced_latency_MA * 1E3
            d["input-pos-interpolMA"] = interpolate_at_timestamp(s, interpolation_timestamp_MA, None)
            d["timeToFrametimeMA"] = d["timeToFrametime"] + introduced_latency_MA
            d["distToFrametimeMA"] = abs(d["input-pos-interpolMA"] - d["output-posMA"])

            # Resampling at frametime with different delays
            for delay in delays:
                predict_time = frametime - delay * 1E3
                for predTech in predTechs:
                    d["resampling-delay-%dms"%delay] = \
                        predict_event_at_timestamp(s, i-1, predict_time, d, pred, predTech)
                    d["resampling-delay-%dms-%s"%(delay, predTech)] = d["resampling-delay-%dms"%delay]
                d["input-pos-interpol-%dms"%delay] = interpolate_at_timestamp(s, predict_time, None)
                d["timeToFrametime-%dms"%delay] = delay
                d["distToFrametime-%dms"%delay] = abs(d["input-pos-interpol"] - d["resampling-delay-%dms"%delay])

            out.append(d)

        last_event = i

        # Compute timestamp for next frame to display
        frametime += (1.0/outFreq) * 1E6


    return out

def processFiles(path) -> None:
    start_time = time.time()
    allRes = []
    allStrokes = []

    files = [f for f in listdir(path) if join(path, f).endswith(".json")]

    for f in files:
        print("Processing %s"%f)

        strokes = []
        # load json file and separate strokes
        with open(join(path, f)) as json_file:
            data = json.load(json_file)
            s = []
            pos = 0
            for p in data:
                if p["type"] == "ScrollBegin":
                    if len(s) > 0:
                        strokes.append(s)
                        s = []
                        pos = 0
                pos += p["delta_y"]
                p['y'] = pos
                s.append(p)

        for s in strokes:
            allStrokes.append(s)

        for outFreq in frequencies:
            d = {}
            r = re.split('-|_',f)
            d["task"] = r[0]
            d["part"] = r[1]
            d["device"] = r[2]
            d["freq"] = outFreq

            outputFreq = float(outFreq)      

            techniques = ["Baseline", "MA"]
            for delay in delays:
                techniques.append("Resampling%sms"%delay)

            metrics = ["Axel"]
            jitters = {}

            timeToFTBaseline = []
            distToFTBaseline = []
            timeToFTMA = []
            distToFTMA = []
            timetoFT = {}
            disttoFT = {}
            for delay in delays:
                timetoFT["timeToFrametime-%dms"%delay] = []
                disttoFT["distToFrametime-%dms"%delay] = []

            for s in strokes:
                if len(s) > 4:
                    out = computeOutputViewPortPos(s, outputFreq)

                    timeToFTBaseline.append(computeAverageValue([o["timeToFrametime"] for o in out]))
                    distToFTBaseline.append(computeAverageValue([o["distToFrametime"] for o in out]))

                    timeToFTMA.append(computeAverageValue([o["timeToFrametimeMA"] for o in out][1:-1]))
                    distToFTMA.append(computeAverageValue([o["distToFrametimeMA"] for o in out][1:-1]))

                    for delay in delays:
                        timetoFT["timeToFrametime-%dms"%delay].append(computeAverageValue([o["timeToFrametime-%dms"%delay] for o in out]))
                        disttoFT["distToFrametime-%dms"%delay].append(computeAverageValue([o["distToFrametime-%dms"%delay] for o in out]))

                    for t in techniques:
                        for m in metrics:
                            tm = t + "-" + m
                            # Initialize list
                            if tm not in jitters.keys():
                                jitters[tm] = []
                            else:
                                dly = re.findall(r'\d+', t)
                                if len(dly) == 0: # Baseline
                                    if m == "Axel":
                                        jitters[tm].append(computeJitterMetricAxelVersion(out, "output-pos","input-pos-interpol"))
                                        if t == "MA":
                                            jitters[tm].append(computeJitterMetricAxelVersion(out[1:-1], "output-posMA","input-pos-interpolMA"))
    
                                else:
                                    delay = int(dly[0])
                                    if m == "Axel":
                                        jitters[tm].append(computeJitterMetricAxelVersion(out, "resampling-delay-%dms"%delay, "input-pos-interpol-%dms"%delay))


            for j in jitters.keys():
                d[j] = computeGlobalJitter(jitters[j])

            d["timeToFT-baseline"] = computeAverageValue(timeToFTBaseline)
            d["distToFT-baseline"] = computeAverageValue(distToFTBaseline)

            d["timeToFT-MA"] = computeAverageValue(timeToFTMA)
            d["distToFT-MA"] = computeAverageValue(distToFTMA)

            for delay in delays:
                d["timeToFT-resamp-delay-%dms"%delay] = computeAverageValue(timetoFT["timeToFrametime-%dms"%delay])
                d["distToFT-resamp-delay-%dms"%delay] = computeAverageValue(disttoFT["distToFrametime-%dms"%delay])


            allRes.append(d)

    keys = allRes[0].keys()
    with open('data.csv', 'w') as output_file:
        dict_writer = csv.DictWriter(output_file, keys)
        dict_writer.writeheader()
        dict_writer.writerows(allRes)

    elapsed_time = time.time() - start_time
    print("Took %s minutes to process."%(elapsed_time / 60.0))

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Scroll predictor simulator')
    parser.add_argument('path', metavar='path', help='path to cleaned json files')
    parser.add_argument('--comp', help = 'Compute actual jitter for Pix 3 and 4', action="store_true")
    args = parser.parse_args()

    if args.comp:
        processFilesComputeActualOutputJitter(args.path, delays)
    else:
        processFiles(args.path)


